use maud::PreEscaped;
use owl::{query, update};

use super::User;
use crate::{auth::Session, ui::prelude::script};
use std::str::FromStr;

pub trait CSRF {
    fn get_csrf(&self) -> impl std::future::Future<Output = uuid::Uuid>;
    fn verify_csrf(&self, csrf: &str) -> impl std::future::Future<Output = bool>;
    fn update_csrf(&self) -> impl std::future::Future<Output = PreEscaped<String>>;
}

impl CSRF for User {
    /// Javascript to update the `value` of an element with id `csrf`.
    ///
    /// This is useful for htmx requests to update the CSRF token in place.
    async fn update_csrf(&self) -> PreEscaped<String> {
        script(&format!(
            "document.querySelectorAll('.csrf').forEach(element => {{ element.value = '{}'; }});",
            self.get_csrf().await
        ))
    }

    /// Get CSRF Token for the current session
    async fn get_csrf(&self) -> uuid::Uuid {
        assert!(!self.session.is_empty());

        let res = query!(|s: &Session| { s.token == self.session });

        res.first().unwrap().read().csrf
    }

    /// Verify CSRF and generate a new one
    async fn verify_csrf(&self, csrf: &str) -> bool {
        if self.get_csrf().await == uuid::Uuid::from_str(csrf).unwrap_or_default() {
            let mut res = query!(|s: &Session| { s.token == self.session });
            update!(&mut res, |s: &mut Session| {
                s.csrf = uuid::Uuid::new_v4();
            });

            return true;
        }

        false
    }
}
