use chrono::Utc;
use owl::{dereference, prelude::*, query, save};
use serde::{Deserialize, Serialize};
use uuid::Uuid;

use crate::gen_random;

use super::{User, UserRole};

#[derive(Debug, Clone)]
#[model]
pub struct Session {
    /// The unique ID of the session token
    pub id: Id,
    /// The generated session token
    pub token: String,
    /// The username associated with the session token
    pub user: IdRef<User>,
    /// Session creation time
    pub created: chrono::DateTime<Utc>,
    /// Internal CSRF value
    pub csrf: uuid::Uuid,
    /// Named session value
    pub name: Option<String>,
    /// Kind of session
    pub kind: SessionKind,
}

#[allow(clippy::upper_case_acronyms)]
#[derive(Debug, Clone, Serialize, Deserialize, sqlx::Type)]
#[sqlx(type_name = "session_kind", rename_all = "lowercase")]
pub enum SessionKind {
    API,
    USER,
}

pub trait Sessions {
    fn from_session(session: String) -> impl std::future::Future<Output = Option<Model<User>>>;
    fn login(
        username: &str,
        password: &str,
    ) -> impl std::future::Future<Output = Option<(Model<Session>, UserRole)>>;
    fn api_key(&self, name: &str) -> impl std::future::Future<Output = Model<Session>>;
    fn session(&self) -> impl std::future::Future<Output = Model<Session>>;
    fn list_sessions(&self) -> impl std::future::Future<Output = Vec<Model<Session>>>;
    fn end_session(&self, id: &uuid::Uuid) -> impl std::future::Future<Output = ()>;
}

impl Sessions for User {
    /// Generate a new API Key session
    async fn api_key(&self, name: &str) -> Model<Session> {
        save!(Session {
            id: Id::new_ulid(),
            token: gen_random(64),
            user: self.reference(),
            created: chrono::Utc::now(),
            csrf: Uuid::new_v4(),
            name: Some(name.to_string()),
            kind: SessionKind::API
        })
    }

    /// End a user session
    async fn end_session(&self, id: &uuid::Uuid) {
        /* TODO : deletion
        sqlx::query("DELETE FROM user_session WHERE id = $1 AND \"user\" = $2")
        .bind(id)
        .bind(&self.username)
        .execute(get_pg!())
        .await
        .unwrap();
        */
    }

    /// Get all sessions for a user
    async fn list_sessions(&self) -> Vec<Model<Session>> {
        query!(|ses: &Session| ses.user.to_string() == self.reference().to_string())
    }

    // Get a user from session ID
    async fn from_session(session_token: String) -> Option<Model<User>> {
        let session = query!(|ses: &Session| ses.token == session_token);
        let session = session.first();
        if let Some(ses) = session {
            let mut user = dereference!(ses.read().user);
            user.write_raw_inline(|u: &mut _| {
                u.session = session_token.to_string();
            });
            return Some(user);
        }

        None
    }

    /// Login a user with the given username and password
    async fn login(username: &str, password: &str) -> Option<(Model<Session>, UserRole)> {
        let u = Self::find(username).await?;
        let u = u.read();

        if !u.verify_pw(password) {
            return None;
        }

        Some((u.session().await, u.user_role.clone()))
    }

    /// Generate a new session token for the user
    ///
    /// Returns a Session instance containing the generated token and associated user
    async fn session(&self) -> Model<Session> {
        save!(Session {
            id: Id::new_ulid(),
            token: gen_random(64),
            user: self.reference(),
            created: chrono::Utc::now(),
            csrf: Uuid::new_v4(),
            name: None,
            kind: SessionKind::USER
        })
    }
}
