#![feature(const_vec_string_slice)]
use data_encoding::HEXUPPER;
use rand::RngCore;
use tokio::sync::OnceCell;

pub mod asset;
pub mod auth;
pub mod format;
pub mod ogp;
pub mod request;
pub mod result;
pub mod ui;

// TODO : CORS?

// Postgres

// TODO : Refactor caching

// TODO : IDEA
// more efficient table join using WHERE ANY instead of multiple SELECTs
// map_tables(Vec<T>, Fn(&T) -> U) -> Vec<U>

pub static PG: OnceCell<sqlx::PgPool> = OnceCell::const_new();

/// A macro to retrieve or initialize the `PostgreSQL` connection pool.
///
/// This macro provides a convenient way to access the `PgPool`. If the pool is not already initialized,
/// it creates a new pool using the connection string from the `$DATABASE_URL` environment variable.
///
/// # Example
/// ```ignore
/// use based::get_pg;
///
/// let pool = get_pg!();
/// ```
#[macro_export]
macro_rules! get_pg {
    () => {
        if let Some(client) = $crate::PG.get() {
            client
        } else {
            let client = sqlx::postgres::PgPoolOptions::new()
                .max_connections(12)
                .connect(&std::env::var("DATABASE_URL").unwrap())
                .await
                .unwrap();
            $crate::PG.set(client).unwrap();
            $crate::PG.get().unwrap()
        }
    };
}

pub fn gen_random(token_length: usize) -> String {
    let mut token_bytes = vec![0u8; token_length];

    rand::thread_rng().fill_bytes(&mut token_bytes);

    HEXUPPER.encode(&token_bytes)
}

fn transpose<T: Clone>(matrix: Vec<Vec<T>>) -> Vec<Vec<T>> {
    if matrix.is_empty() {
        return vec![];
    }

    let row_len = matrix[0].len();
    let mut transposed = vec![Vec::with_capacity(matrix.len()); row_len];

    for row in matrix {
        for (i, elem) in row.into_iter().enumerate() {
            transposed[i].push(elem);
        }
    }

    transposed
}

// TODO : More types
#[derive(Clone)]
pub enum DatabaseType {
    TEXT(String),
    INTEGER(i32),
    BIGINT(i64),
    REAL(f32),
    DOUBLE(f64),
    BOOLEAN(bool),
    BYTEA(Vec<u8>),
    UUID(uuid::Uuid),
    JSON(serde_json::Value),
}

impl DatabaseType {
    pub fn type_name(&self) -> &str {
        match self {
            Self::TEXT(_) => "TEXT",
            Self::INTEGER(_) => "INTEGER",
            Self::BIGINT(_) => "BIGINT",
            Self::REAL(_) => "REAL",
            Self::DOUBLE(_) => "DOUBLE PRECISION",
            Self::BOOLEAN(_) => "BOOLEAN",
            Self::BYTEA(_) => "BYTEA",
            Self::UUID(_) => "UUID",
            Self::JSON(_) => "JSONB",
        }
    }

    pub fn as_json(self) -> serde_json::Value {
        match self {
            DatabaseType::JSON(result) => return result,
            _ => panic!["Wrong DB type"],
        }
    }

    pub fn as_uuid(self) -> uuid::Uuid {
        match self {
            DatabaseType::UUID(result) => return result,
            _ => panic!["Wrong DB type"],
        }
    }

    pub fn as_vec(self) -> Vec<u8> {
        match self {
            DatabaseType::BYTEA(result) => return result,
            _ => panic!["Wrong DB type"],
        }
    }

    pub fn as_bool(self) -> bool {
        match self {
            DatabaseType::BOOLEAN(result) => return result,
            _ => panic!["Wrong DB type"],
        }
    }

    pub fn as_f64(self) -> f64 {
        match self {
            DatabaseType::DOUBLE(result) => return result,
            _ => panic!["Wrong DB type"],
        }
    }

    pub fn as_f32(self) -> f32 {
        match self {
            DatabaseType::REAL(result) => return result,
            _ => panic!["Wrong DB type"],
        }
    }

    pub fn as_big_int(self) -> i64 {
        match self {
            DatabaseType::BIGINT(result) => return result,
            _ => panic!["Wrong DB type"],
        }
    }

    pub fn as_integer(self) -> i32 {
        match self {
            DatabaseType::INTEGER(result) => return result,
            _ => panic!("Wrong DB type"),
        }
    }

    pub fn as_text(self) -> String {
        match self {
            DatabaseType::TEXT(result) => return result,
            _ => panic!("Wrong DB type"),
        }
    }
}

// TODO : fix errors on conflict -> ignore already inserted ones
pub async fn batch_insert(table: &str, values: &[String], entries: Vec<Vec<DatabaseType>>) {
    assert_eq!(values.len(), entries.first().unwrap().len());

    let mut cmd = format!("INSERT INTO {table} (");

    cmd.push_str(&values.join(", "));

    cmd.push_str(") SELECT * FROM UNNEST(");

    let entries = transpose(entries);

    for i in 0..values.len() {
        let t = entries.get(i).unwrap().first().unwrap().type_name();
        if i == (entries.len() - 1) {
            cmd.push_str(&format!("${}::{t}[]", i + 1));
        } else {
            cmd.push_str(&format!("${}::{t}[],", i + 1));
        }
    }

    cmd.push_str(")");

    log::debug!("Executing batch query: {cmd}");

    let mut query = sqlx::query(&cmd);

    for e in entries {
        let first = e.first().unwrap();
        match first {
            DatabaseType::TEXT(_) => {
                query = query.bind(e.into_iter().map(|x| x.as_text()).collect::<Vec<_>>());
            }
            DatabaseType::INTEGER(_) => {
                query = query.bind(e.into_iter().map(|x| x.as_integer()).collect::<Vec<_>>());
            }
            DatabaseType::BIGINT(_) => {
                query = query.bind(e.into_iter().map(|x| x.as_big_int()).collect::<Vec<_>>());
            }
            DatabaseType::REAL(_) => {
                query = query.bind(e.into_iter().map(|x| x.as_f32()).collect::<Vec<_>>());
            }
            DatabaseType::DOUBLE(_) => {
                query = query.bind(e.into_iter().map(|x| x.as_f64()).collect::<Vec<_>>());
            }
            DatabaseType::BOOLEAN(_) => {
                query = query.bind(e.into_iter().map(|x| x.as_bool()).collect::<Vec<_>>());
            }
            DatabaseType::BYTEA(_) => {
                query = query.bind(e.into_iter().map(|x| x.as_vec()).collect::<Vec<_>>());
            }
            DatabaseType::UUID(_) => {
                query = query.bind(e.into_iter().map(|x| x.as_uuid()).collect::<Vec<_>>());
            }
            DatabaseType::JSON(_) => {
                query = query.bind(e.into_iter().map(|x| x.as_json()).collect::<Vec<_>>());
            }
        }
    }

    query.execute(get_pg!()).await.unwrap();
}

// TODO : LibraryIndex
// index, cleanup, add_one, remove_one, get, exists, query
