#[allow(non_snake_case)]
pub fn Popover<T: UIWidget + 'static, I: UIWidget + 'static>(on: T, inner: I) -> PopoverWidget {
    PopoverWidget {
        on: Box::new(on),
        inner: Box::new(inner),
        placement: Placement::Top,
        arrow: true,
        animated: true,
        trigger: PopoverTrigger::Hover,
        offset: 10,
    }
}

pub enum PopoverTrigger {
    Click,
    Hover,
}

pub struct PopoverWidget {
    on: Box<dyn UIWidget>,
    inner: Box<dyn UIWidget>,
    placement: Placement,
    arrow: bool,
    animated: bool,
    trigger: PopoverTrigger,
    offset: i32,
}

impl PopoverWidget {
    pub fn place(mut self, placement: Placement) -> Self {
        self.placement = placement;
        self
    }

    pub fn no_arrow(mut self) -> Self {
        self.arrow = false;
        self
    }

    pub fn animate(mut self, value: bool) -> Self {
        self.animated = value;
        self
    }

    pub fn trigger(mut self, trigger: PopoverTrigger) -> Self {
        self.trigger = trigger;
        self
    }

    pub fn offset(mut self, offset: i32) -> Self {
        self.offset = offset;
        self
    }
}

impl Render for PopoverWidget {
    fn render(&self) -> maud::Markup {
        self.render_with_class("")
    }
}

impl UIWidget for PopoverWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn extended_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn render_with_class(&self, class: &str) -> maud::Markup {
        let id = format!("popover-{}", uuid::Uuid::new_v4().to_string());

        let el_class = "absolute z-10 invisible inline-block w-fit p-2 text-sm text-gray-500 bg-white border border-gray-200 rounded-lg shadow-xs opacity-0 dark:text-gray-400 dark:border-gray-600 dark:bg-gray-800";

        let el_class = if self.animated {
            format!("{el_class} transition-opacity duration-200")
        } else {
            el_class.to_string()
        };

        let trigger = match self.trigger {
            PopoverTrigger::Click => "click",
            PopoverTrigger::Hover => "hover",
        };

        html! {

        div data-popover-target=(id)
        data-popover-trigger=(trigger)
        data-popover-placement=(self.placement.to_value())
        data-popover-offset=(self.offset)
        class=(format!("{class} my-auto max-w-fit")) { (self.on) };

        div data-popover id=(id) role="tooltip" class=(el_class) {
            (self.inner)

            @if self.arrow {
            div data-popper-arrow {};
            }
        }

                    }
    }
}

use maud::{PreEscaped, Render, html};

use crate::ui::UIWidget;

#[allow(non_snake_case)]
pub fn Tooltip<T: UIWidget + 'static, I: UIWidget + 'static>(on: T, inner: I) -> TooltipWidget {
    TooltipWidget {
        on: Box::new(on),
        inner: Box::new(inner),
        placement: Placement::Top,
        arrow: true,
        dark: true,
        animated: true,
    }
}

pub enum Placement {
    Left,
    Right,
    Top,
    Bottom,
}

impl Placement {
    pub fn to_value(&self) -> &str {
        match *self {
            Placement::Left => "left",
            Placement::Right => "right",
            Placement::Top => "top",
            Placement::Bottom => "bottom",
        }
    }
}

pub struct TooltipWidget {
    on: Box<dyn UIWidget>,
    inner: Box<dyn UIWidget>,
    placement: Placement,
    arrow: bool,
    dark: bool,
    animated: bool,
}

impl TooltipWidget {
    pub fn place(mut self, placement: Placement) -> Self {
        self.placement = placement;
        self
    }

    pub fn no_arrow(mut self) -> Self {
        self.arrow = false;
        self
    }

    pub fn animate(mut self, value: bool) -> Self {
        self.animated = value;
        self
    }

    pub fn white(mut self) -> Self {
        self.dark = false;
        self
    }
}

impl Render for TooltipWidget {
    fn render(&self) -> maud::Markup {
        self.render_with_class("")
    }
}

impl UIWidget for TooltipWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn extended_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn render_with_class(&self, class: &str) -> maud::Markup {
        let id = format!("tooltip-{}", uuid::Uuid::new_v4().to_string());

        let tt_class = match self.dark {
            true => {
                "absolute z-10 invisible inline-block px-3 py-2 text-sm font-medium text-white bg-gray-900 rounded-lg shadow-xs opacity-0 tooltip dark:bg-gray-700"
            }
            false => {
                "absolute z-10 invisible inline-block px-3 py-2 text-sm font-medium text-gray-900 bg-white border border-gray-200 rounded-lg shadow-xs opacity-0 tooltip"
            }
        };

        let tt_class = if self.animated {
            format!("{tt_class} transition-opacity duration-200")
        } else {
            tt_class.to_string()
        };

        html! {
            @if self.dark {
                div data-tooltip-target=(id) data-tooltip-placement=(self.placement.to_value())
                class=(format!("{class} my-auto max-w-fit")) { (self.on) };
            } @else {
                div data-tooltip-target=(id) data-tooltip-style="light" data-tooltip-placement=(self.placement.to_value())
                class=(format!("{class} my-auto max-w-fit")) { (self.on) };
            }

            div id=(id) role="tooltip" class=(tt_class) {
                (self.inner)
                @if self.arrow {
                    div class="tooltip-arrow" data-popper-arrow {};
                }
        }}
    }
}

#[allow(non_snake_case)]
pub fn DropDown<T: UIWidget + 'static, I: UIWidget + 'static>(on: T, inner: I) -> DropDownWidget {
    DropDownWidget {
        on: Box::new(on),
        inner: Box::new(inner),
        delay: 0,
        placement: Placement::Bottom,
        trigger: PopoverTrigger::Click,
        distance: 10,
        skidding: 0,
        stretch: false,
    }
}

pub struct DropDownWidget {
    on: Box<dyn UIWidget>,
    inner: Box<dyn UIWidget>,
    placement: Placement,
    delay: u32,
    trigger: PopoverTrigger,
    distance: u32,
    skidding: u32,
    stretch: bool,
}

impl DropDownWidget {
    pub fn place(mut self, placement: Placement) -> Self {
        self.placement = placement;
        self
    }

    pub fn delay(mut self, delay: u32) -> Self {
        self.delay = delay;
        self
    }

    pub fn stretch(mut self) -> Self {
        self.stretch = true;
        self
    }

    pub fn distance(mut self, distance: u32) -> Self {
        self.distance = distance;
        self
    }

    pub fn skidding(mut self, skidding: u32) -> Self {
        self.skidding = skidding;
        self
    }

    pub fn trigger(mut self, trigger: PopoverTrigger) -> Self {
        self.trigger = trigger;
        self
    }
}

impl Render for DropDownWidget {
    fn render(&self) -> maud::Markup {
        self.render_with_class("")
    }
}

impl UIWidget for DropDownWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn extended_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn render_with_class(&self, class: &str) -> maud::Markup {
        let id = format!("dropdown-{}", uuid::Uuid::new_v4().to_string());

        let trigger = match self.trigger {
            PopoverTrigger::Click => "click",
            PopoverTrigger::Hover => "hover",
        };

        html! {
            div
            data-dropdown-toggle=(id)
            data-dropdown-trigger=(trigger)
            data-dropdown-delay=(self.delay)
            data-dropdown-placement=(self.placement.to_value())
            data-dropdown-offset-distance=(self.distance)
            data-dropdown-offset-skidding=(self.skidding)
            class=(format!("{class} hover:cursor-pointer my-auto{}", if self.stretch { "" } else { " max-w-fit" })) {
                (self.on)
            };

            div id=(id) class="z-50 hidden bg-white divide-y divide-gray-100 rounded-lg shadow-sm w-44 dark:bg-gray-700" {
                ul class="py-2 text-sm text-gray-700 dark:text-gray-200" {
                    (self.inner)
                }
            }
        }
    }
}

#[allow(non_snake_case)]
pub fn DropdownMenuEntry<T: UIWidget + 'static>(link: &str, inner: T) -> PreEscaped<String> {
    html! {
        li {
            a href=(link) class="block px-4 py-2 hover:bg-gray-100 dark:hover:bg-gray-600 dark:hover:text-white" { (inner) }
        }
    }
}

#[allow(non_snake_case)]
pub fn DropdownMenu(entries: Vec<PreEscaped<String>>) -> PreEscaped<String> {
    html! {
        ul class="py-1 text-sm text-gray-700 dark:text-gray-200" {
            @for e in entries {
                (e)
            }
        }
    }
}
