use maud::{PreEscaped, Render, html};

use crate::{request::api::Pager, ui::UIWidget};

#[allow(non_snake_case)]
pub fn Pagination<T, F>(
    current_page: usize,
    pager: Pager<T>,
    url_builder: F,
) -> PaginationWidget<T, F>
where
    F: Fn(usize) -> String,
{
    PaginationWidget {
        icons: true,
        pager,
        current_page,
        url_builder,
    }
}

pub struct PaginationWidget<T, F>
where
    F: Fn(usize) -> String,
{
    icons: bool,
    pager: Pager<T>,
    current_page: usize,
    url_builder: F,
}

impl<T, F> PaginationWidget<T, F>
where
    F: Fn(usize) -> String,
{
    pub fn no_icons(mut self) -> Self {
        self.icons = false;
        self
    }

    fn build_page_link_active(page_num: usize, link: &str, last: bool) -> PreEscaped<String> {
        html! {
            li {
                a href=(link) aria-current="page" class=(
                    format!("flex items-center justify-center px-3 h-8 text-blue-600 border border-gray-300 bg-blue-50 hover:bg-blue-100 hover:text-blue-700 dark:border-gray-700 dark:bg-gray-700 dark:text-white {} {}",
                    if page_num == 1 { "rounded-s-lg" } else { "" },
                    if last { "rounded-e-lg" } else { "" }
                )) { (page_num) }
            }
        }
    }

    fn build_page_link(page_num: usize, link: &str, last: bool) -> PreEscaped<String> {
        html! {
            li {
                a href=(link) class=
                (format!("flex items-center justify-center px-3 h-8 leading-tight text-gray-500 bg-white border border-gray-300 hover:bg-gray-100 hover:text-gray-700 dark:bg-gray-800 dark:border-gray-700 dark:text-gray-400 dark:hover:bg-gray-700 dark:hover:text-white {}", if last { "rounded-e-lg" } else { "" })) { (page_num) }
            }
        }
    }

    fn build_page_links(&self, current_page: usize) -> PreEscaped<String> {
        let total_pages = self.pager.total_pages();
        let mut start_page = if current_page <= 3 {
            1
        } else {
            current_page - 2
        };
        let mut end_page = if total_pages - current_page < 2 {
            total_pages
        } else {
            current_page + 2
        };

        if end_page - start_page < 4 {
            if start_page > 1 {
                start_page = start_page.saturating_sub(4 - (end_page - start_page));
            } else {
                end_page = (start_page + 4).min(total_pages);
            }
        }

        html! {
            @for page in start_page..=end_page {
                @if page == current_page {
                    (Self::build_page_link_active(page, &(self.url_builder)(page), current_page == total_pages && page == current_page))
                } @else {
                    (Self::build_page_link(page, &(self.url_builder)(page), current_page == total_pages && page == current_page))
                }
            }
        }
    }

    fn build(&self, page: usize) -> PreEscaped<String> {
        html! {
                nav {
                ul class="inline-flex -space-x-px text-sm" {
                    @if page != 1 {
                  li {
                    a href=((self.url_builder)(page-1)) class="flex items-center justify-center px-3 h-8 ms-0 leading-tight text-gray-500 bg-white border border-e-0 border-gray-300 rounded-s-lg hover:bg-gray-100 hover:text-gray-700 dark:bg-gray-800 dark:border-gray-700 dark:text-gray-400 dark:hover:bg-gray-700 dark:hover:text-white" {
                      @if self.icons {
                        span class="sr-only" { "Previous" };
                      svg class="w-2.5 h-2.5 rtl:rotate-180" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 6 10" {
                        path stroke="currentColor" stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M5 1 1 5l4 4" {};
                    }
                } @else {
                    "Previous"
                }

            };
        }
                  }

            (self.build_page_links(page))


            @if self.pager.has_next_page(page as u64) {
                li {
                    a href=((self.url_builder)(page+1)) class="flex items-center justify-center px-3 h-8 leading-tight text-gray-500 bg-white border border-gray-300 rounded-e-lg hover:bg-gray-100 hover:text-gray-700 dark:bg-gray-800 dark:border-gray-700 dark:text-gray-400 dark:hover:bg-gray-700 dark:hover:text-white" {

                      @if self.icons {
                      span class="sr-only" { "Next" };
                    svg class="w-2.5 h-2.5 rtl:rotate-180" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 6 10" {
                      path stroke="currentColor" stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="m1 9 4-4-4-4" {};
                    }
                } @else {
                    "Next"
                }

                     }
                }
            };

                };
              };

              }
    }
}

impl<T, F> Render for PaginationWidget<T, F>
where
    F: Fn(usize) -> String,
{
    fn render(&self) -> maud::Markup {
        self.render_with_class("")
    }
}

impl<T, F> UIWidget for PaginationWidget<T, F>
where
    F: Fn(usize) -> String,
{
    fn can_inherit(&self) -> bool {
        false
    }

    fn base_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn extended_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn render_with_class(&self, _: &str) -> maud::Markup {
        self.build(self.current_page)
    }
}

#[allow(non_snake_case)]
pub fn PaginationButtonsOnly<T, F>(
    page: u64,
    pager: &Pager<T>,
    url_builder: F,
) -> PreEscaped<String>
where
    F: Fn(usize) -> String,
{
    let previous = page > 1;
    let next = pager.page_with_context(page).1;

    let buttons = html! {
            @if previous {
                a href=(url_builder((page-1) as usize)) class="flex items-center justify-center px-3 h-8 me-3 text-sm font-medium text-gray-500 bg-white border border-gray-300 rounded-lg hover:bg-gray-100 hover:text-gray-700 dark:bg-gray-800 dark:border-gray-700 dark:text-gray-400 dark:hover:bg-gray-700 dark:hover:text-white" {
                    svg class="w-3.5 h-3.5 me-2 rtl:rotate-180" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 14 10" {
                        path stroke="currentColor" stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M13 5H1m0 0 4 4M1 5l4-4" {};
                    }
                    "Previous"
                }
            };

            @if next {
                a href=(url_builder((page+1) as usize)) class="flex items-center justify-center px-3 h-8 text-sm font-medium text-gray-500 bg-white border border-gray-300 rounded-lg hover:bg-gray-100 hover:text-gray-700 dark:bg-gray-800 dark:border-gray-700 dark:text-gray-400 dark:hover:bg-gray-700 dark:hover:text-white" {
                    "Next"
        svg class="w-3.5 h-3.5 ms-2 rtl:rotate-180" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 14 10" {
            path stroke="currentColor" stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M1 5h12m0 0L9 1m4 4L9 9" {};
        }
    };
    }};

    html! {
        @if previous {
        div class="flex justify-between" {
            (buttons)
        };
    } @else {
        div class="flex justify-end" {
            (buttons)
        };
    }
    }
}
