use crate::ui::UIWidget;
use maud::{Markup, Render, html};

use super::Size;

#[allow(non_snake_case)]
pub fn Blur<T: UIWidget + 'static>(amount: Size, inner: T) -> BlurWidget {
    BlurWidget(Box::new(inner), amount, false)
}

pub struct BlurWidget(Box<dyn UIWidget>, Size, bool);

impl BlurWidget {
    pub fn backdrop(mut self) -> Self {
        self.2 = true;
        self
    }
}

impl Render for BlurWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for BlurWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        let class = match &self.1 {
            Size::Custom(s) => &format!(" blur-[{s}]"),
            Size::None => "blur-none",
            Size::Small => "blur-sm",
            Size::Regular => "blur",
            Size::Medium => "blur-md",
            Size::Large => "blur-lg",
            Size::XL => "blur-xl",
            Size::_2XL => "blur-2xl",
            Size::_3XL => "blur-3xl",
            Size::Full => "blur-3xl",
        };

        if self.2 {
            return vec![format!("backdrop-{class}")];
        }

        vec![class.to_string()]
    }

    fn extended_class(&self) -> Vec<String> {
        let mut c = self.base_class();
        c.extend_from_slice(&self.0.extended_class());
        c
    }

    fn render_with_class(&self, class: &str) -> Markup {
        if self.0.as_ref().can_inherit() {
            self.0
                .as_ref()
                .render_with_class(&format!("{} {class}", self.base_class().join(" ")))
        } else {
            html! {
                div class=(format!("{} {class}", self.base_class().join(" "))) {
                    (self.0.as_ref())
                }
            }
        }
    }
}

macro_rules! build_value_widget {
    ($constr:ident, $widget:ident, $class:literal) => {
        #[allow(non_snake_case)]
        pub fn $constr<T: UIWidget + 'static>(value: f64, inner: T) -> $widget {
            $widget(Box::new(inner), value, false)
        }

        pub struct $widget(Box<dyn UIWidget>, f64, bool);

        impl $widget {
            pub fn backdrop(mut self) -> Self {
                self.2 = true;
                self
            }
        }

        impl Render for $widget {
            fn render(&self) -> Markup {
                self.render_with_class("")
            }
        }

        impl UIWidget for $widget {
            fn can_inherit(&self) -> bool {
                true
            }

            fn base_class(&self) -> Vec<String> {
                let mut ret = $class.to_string();
                ret.push_str(&format!("-[{:.2}]", self.1));

                if self.2 {
                    return vec![format!("backdrop-{ret}")];
                }

                vec![ret]
            }

            fn extended_class(&self) -> Vec<String> {
                let mut c = self.base_class();
                c.extend_from_slice(&self.0.extended_class());
                c
            }

            fn render_with_class(&self, class: &str) -> Markup {
                if self.0.as_ref().can_inherit() {
                    self.0
                        .as_ref()
                        .render_with_class(&format!("{} {class}", self.base_class().join(" ")))
                } else {
                    html! {
                        div class=(format!("{} {class}", self.base_class().join(" "))) {
                            (self.0.as_ref())
                        }
                    }
                }
            }
        }
    };
}

build_value_widget!(Brightness, BrightnessWidget, "brightness");
build_value_widget!(Contrast, ConstrastWidget, "contrast");
build_value_widget!(Saturate, SaturationWidget, "saturate");

macro_rules! build_on_off_widget {
    ($constr:ident, $widget:ident, $class:literal) => {
        #[allow(non_snake_case)]
        pub fn $constr<T: UIWidget + 'static>(inner: T) -> $widget {
            $widget(Box::new(inner), true, false)
        }

        pub struct $widget(Box<dyn UIWidget>, bool, bool);

        impl $widget {
            pub fn none(mut self) -> Self {
                self.1 = false;
                self
            }

            pub fn backdrop(mut self) -> Self {
                self.2 = true;
                self
            }
        }

        impl Render for $widget {
            fn render(&self) -> Markup {
                self.render_with_class("")
            }
        }

        impl UIWidget for $widget {
            fn can_inherit(&self) -> bool {
                true
            }

            fn base_class(&self) -> Vec<String> {
                let class = if self.1 {
                    $class.to_string()
                } else {
                    concat!($class, "-0").to_string()
                };

                if self.2 {
                    return vec![format!("backdrop-{class}")];
                }

                vec![class]
            }

            fn extended_class(&self) -> Vec<String> {
                let mut c = self.base_class();
                c.extend_from_slice(&self.0.extended_class());
                c
            }

            fn render_with_class(&self, class: &str) -> Markup {
                if self.0.as_ref().can_inherit() {
                    self.0
                        .as_ref()
                        .render_with_class(&format!("{} {class}", self.base_class().join(" ")))
                } else {
                    html! {
                        div class=(format!("{} {class}", self.base_class().join(" "))) {
                            (self.0.as_ref())
                        }
                    }
                }
            }
        }
    };
}

build_on_off_widget!(Grayscale, GrayscaleWidget, "grayscale");
build_on_off_widget!(Invert, InvertWidget, "invert");
build_on_off_widget!(Sepia, SepiaWidget, "sepia");

#[allow(non_snake_case)]
pub fn HueRotate<T: UIWidget + 'static>(deg: u32, inner: T) -> HueRotateWidget {
    HueRotateWidget(Box::new(inner), deg, false)
}

pub struct HueRotateWidget(Box<dyn UIWidget>, u32, bool);

impl HueRotateWidget {
    pub fn backdrop(mut self) -> Self {
        self.2 = true;
        self
    }
}

impl Render for HueRotateWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for HueRotateWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        let class = format!("hue-rotate-[{:.2}deg]", self.1);

        if self.2 {
            return vec![format!("backdrop-{class}")];
        }

        vec![class]
    }

    fn extended_class(&self) -> Vec<String> {
        let mut c = self.base_class();
        c.extend_from_slice(&self.0.extended_class());
        c
    }

    fn render_with_class(&self, class: &str) -> Markup {
        if self.0.as_ref().can_inherit() {
            self.0
                .as_ref()
                .render_with_class(&format!("{} {class}", self.base_class().join(" ")))
        } else {
            html! {
                div class=(format!("{} {class}", self.base_class().join(" "))) {
                    (self.0.as_ref())
                }
            }
        }
    }
}

#[allow(non_snake_case)]
pub fn Opacity<T: UIWidget + 'static>(value: f64, inner: T) -> OpacityWidget {
    OpacityWidget(Box::new(inner), value, false)
}

pub struct OpacityWidget(Box<dyn UIWidget>, f64, bool);

impl OpacityWidget {
    pub fn backdrop(mut self) -> Self {
        self.2 = true;
        self
    }
}

impl Render for OpacityWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for OpacityWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        let class = match self.1 {
            0.0 => "opacity-0",
            0.05 => "opacity-5",
            0.1 => "opacity-10",
            0.15 => "opacity-15",
            0.2 => "opacity-20",
            0.25 => "opacity-25",
            0.3 => "opacity-30",
            0.35 => "opacity-35",
            0.4 => "opacity-40",
            0.45 => "opacity-45",
            0.5 => "opacity-50",
            0.55 => "opacity-55",
            0.6 => "opacity-60",
            0.65 => "opacity-65",
            0.7 => "opacity-70",
            0.75 => "opacity-75",
            0.8 => "opacity-80",
            0.85 => "opacity-85",
            0.9 => "opacity-90",
            0.95 => "opacity-95",
            1.0 => "opacity-100",
            _ => &format!("opacity-[{:.2}]", self.1),
        };

        if self.2 {
            return vec![format!("backdrop-{class}")];
        }

        vec![class.to_string()]
    }

    fn extended_class(&self) -> Vec<String> {
        let mut c = self.base_class();
        c.extend_from_slice(&self.0.extended_class());
        c
    }

    fn render_with_class(&self, class: &str) -> Markup {
        if self.0.as_ref().can_inherit() {
            self.0
                .as_ref()
                .render_with_class(&format!("{} {class}", self.base_class().join(" ")))
        } else {
            html! {
                div class=(format!("{} {class}", self.base_class().join(" "))) {
                    (self.0.as_ref())
                }
            }
        }
    }
}

pub enum BlendMode {
    Normal,
    Multiply,
    Screen,
    Overlay,
    Darken,
    Lighten,
    ColorDodge,
    ColorBurn,
    HardLight,
    SoftLight,
    Difference,
    Exclusion,
    Hue,
    Saturation,
    Color,
    Luminosity,
    PlusDarker,
    PlusLighter,
}

impl BlendMode {
    pub const fn to_value(&self) -> &str {
        match self {
            BlendMode::Normal => "normal",
            BlendMode::Multiply => "multiply",
            BlendMode::Screen => "screen",
            BlendMode::Overlay => "overlay",
            BlendMode::Darken => "darken",
            BlendMode::Lighten => "lighten",
            BlendMode::ColorDodge => "color-dodge",
            BlendMode::ColorBurn => "color-burn",
            BlendMode::HardLight => "hard-light",
            BlendMode::SoftLight => "soft-light",
            BlendMode::Difference => "difference",
            BlendMode::Exclusion => "exclusion",
            BlendMode::Hue => "hue",
            BlendMode::Saturation => "saturation",
            BlendMode::Color => "color",
            BlendMode::Luminosity => "luminosity",
            BlendMode::PlusDarker => "plus-darker",
            BlendMode::PlusLighter => "plus-lighter",
        }
    }
}

#[allow(non_snake_case)]
pub fn MixBlendMode<T: UIWidget + 'static>(mode: BlendMode, inner: T) -> MixBlendModeWidget {
    MixBlendModeWidget(Box::new(inner), mode)
}

pub struct MixBlendModeWidget(Box<dyn UIWidget>, BlendMode);

impl Render for MixBlendModeWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for MixBlendModeWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        vec![format!("mix-blend-{}", self.1.to_value())]
    }

    fn extended_class(&self) -> Vec<String> {
        let mut c = self.base_class();
        c.extend_from_slice(&self.0.extended_class());
        c
    }

    fn render_with_class(&self, class: &str) -> Markup {
        if self.0.as_ref().can_inherit() {
            self.0
                .as_ref()
                .render_with_class(&format!("{} {class}", self.base_class().join(" ")))
        } else {
            html! {
                div class=(format!("{} {class}", self.base_class().join(" "))) {
                    (self.0.as_ref())
                }
            }
        }
    }
}

#[allow(non_snake_case)]
pub fn BackgroundBlendMode<T: UIWidget + 'static>(
    mode: BlendMode,
    inner: T,
) -> BackgroundBlendModeWidget {
    BackgroundBlendModeWidget(Box::new(inner), mode)
}

pub struct BackgroundBlendModeWidget(Box<dyn UIWidget>, BlendMode);

impl Render for BackgroundBlendModeWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for BackgroundBlendModeWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        vec![format!("bg-blend-{}", self.1.to_value())]
    }

    fn extended_class(&self) -> Vec<String> {
        let mut c = self.base_class();
        c.extend_from_slice(&self.0.extended_class());
        c
    }

    fn render_with_class(&self, class: &str) -> Markup {
        if self.0.as_ref().can_inherit() {
            self.0
                .as_ref()
                .render_with_class(&format!("{} {class}", self.base_class().join(" ")))
        } else {
            html! {
                div class=(format!("{} {class}", self.base_class().join(" "))) {
                    (self.0.as_ref())
                }
            }
        }
    }
}
