use std::collections::HashMap;

use maud::{Markup, PreEscaped, Render, html};

pub mod form;
pub mod toggle;
pub use form::*;
pub use toggle::*;

use crate::ui::{
    AttrExtendable, UIWidget,
    color::{ColorCircle, UIColor},
    htmx::HTMXAttributes,
    prelude::{Nothing, script},
};

#[allow(non_snake_case)]
pub fn TextInput(name: &str) -> TextInputWidget {
    TextInputWidget {
        attrs: HashMap::new(),
        password: false,
        icon: None,
        icon_border: false,
    }
    .name(name)
}

pub struct TextInputWidget {
    attrs: HashMap<String, String>,
    password: bool,
    icon: Option<Box<dyn UIWidget>>,
    icon_border: bool,
}

impl TextInputWidget {
    pub fn placeholder(self, placeholder: &str) -> Self {
        self.add_attr("placeholder", placeholder)
    }

    pub fn pattern(self, pattern: &str) -> Self {
        self.add_attr("pattern", pattern)
    }

    pub fn maxlength(self, maxlength: &str) -> Self {
        self.add_attr("maxlength", maxlength)
    }

    pub fn minlength(self, minlength: &str) -> Self {
        self.add_attr("minlength", minlength)
    }

    pub fn icon_border(mut self) -> Self {
        self.icon_border = true;
        self
    }

    pub fn icon<T: UIWidget + 'static>(mut self, icon: T) -> Self {
        self.icon = Some(Box::new(icon));
        self
    }

    pub fn password(mut self) -> Self {
        self.password = true;
        self
    }
}

impl InputAttr for TextInputWidget {}

impl AttrExtendable for TextInputWidget {
    fn add_attr(mut self, key: &str, val: &str) -> Self {
        self.attrs.insert(key.into(), val.into());
        self
    }
}

impl Render for TextInputWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for TextInputWidget {
    fn can_inherit(&self) -> bool {
        false
    }

    fn base_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn extended_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn render_with_class(&self, _: &str) -> Markup {
        let mut attrs = self.attrs.clone();

        if self.password {
            attrs.insert("type".into(), "password".into());
        } else {
            attrs.insert("type".into(), "text".into());
        }

        if self.icon_border {
            attrs.insert("class".into(), 
        "rounded-none rounded-e-lg bg-gray-50 border text-gray-900 focus:ring-blue-500 focus:border-blue-500 block flex-1 min-w-0 w-full text-sm border-gray-300 p-2.5  dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-blue-500 dark:focus:border-blue-500".into()
        );
        } else {
            attrs.insert("class".into(), 
            format!("bg-gray-50 border border-gray-300 text-gray-900 text-sm rounded-lg focus:ring-blue-500 focus:border-blue-500 block w-full {} p-2.5  dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-blue-500 dark:focus:border-blue-500", if self.icon.is_some() { "ps-10" } else { "" })
        );
        }

        let input = build_element("input", &attrs, Nothing());

        html! {
                    @if self.icon_border {
                        div class="flex" {
                            @if let Some(icon) = &self.icon {
                            span class="inline-flex items-center px-3 text-sm text-gray-900 bg-gray-200 border rounded-e-0 border-gray-300 border-e-0 rounded-s-md dark:bg-gray-600 dark:text-gray-400 dark:border-gray-600" {
                                (icon)
                            };
                        };

        (input)

                        };
                    } @else {
                        div class="relative mb-6" {
                            @if let Some(icon) = &self.icon {
                            div class="absolute inset-y-0 start-0 flex items-center ps-3.5 pointer-events-none" {
                                (icon)
                            };
                        };

                            (input)

                        };
                    }
                }
    }
}

#[allow(non_snake_case)]
pub fn FileInput(name: &str) -> FileInputWidget {
    FileInputWidget {
        dropzone: None,
        attrs: HashMap::new(),
    }
    .name(name)
}

pub struct FileInputWidget {
    dropzone: Option<PreEscaped<String>>,
    attrs: HashMap<String, String>,
}

impl FileInputWidget {
    pub fn multiple(self) -> Self {
        self.add_attr("multiple", "")
    }

    pub fn accept(self, mime: &str) -> Self {
        self.add_attr("accept", mime)
    }

    pub fn dropzone<T: UIWidget + 'static>(mut self, inner: T) -> Self {
        self.dropzone = Some(inner.render());
        self
    }
}

impl InputAttr for FileInputWidget {}

impl AttrExtendable for FileInputWidget {
    fn add_attr(mut self, key: &str, val: &str) -> Self {
        self.attrs.insert(key.into(), val.into());
        self
    }
}

impl Render for FileInputWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for FileInputWidget {
    fn can_inherit(&self) -> bool {
        false
    }

    fn base_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn extended_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn render_with_class(&self, class: &str) -> Markup {
        let class = format!("
        {class} block w-full text-sm text-gray-900 border border-gray-300 rounded-lg cursor-pointer bg-gray-50 dark:text-gray-400 focus:outline-none dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400
        ");

        let mut attrs = self.attrs.clone();

        if self.dropzone.is_some() {
            attrs.insert("class".to_string(), "hidden".to_string());
        } else {
            attrs.insert("class".to_string(), class);
        }
        attrs.insert("type".to_string(), "file".to_string());

        let input_name = attrs.get("name").map(|x| x.as_str()).unwrap_or_default();

        if let Some(dropzone) = &self.dropzone {
            return html! {
                div class="flex items-center justify-center w-full" {
                    label for=(input_name) class="flex flex-col items-center justify-center w-full h-64 border-2 border-gray-300 border-dashed rounded-lg cursor-pointer bg-gray-50 dark:hover:bg-gray-800 dark:bg-gray-700 hover:bg-gray-100 dark:border-gray-600 dark:hover:border-gray-500 dark:hover:bg-gray-600" {
                        div class="flex flex-col items-center justify-center pt-5 pb-6" {
                            svg class="w-8 h-8 mb-4 text-gray-500 dark:text-gray-400" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 20 16" {
                                path stroke="currentColor" stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M13 13h3a3 3 0 0 0 0-6h-.025A5.56 5.56 0 0 0 16 6.5 5.5 5.5 0 0 0 5.207 5.021C5.137 5.017 5.071 5 5 5a4 4 0 0 0 0 8h2.167M10 15V6m0 0L8 8m2-2 2 2" {};
                            };
                            p class="mb-2 text-sm text-gray-500 dark:text-gray-400" { span class="font-semibold" { "Click to upload" } };
                            (dropzone)
                        };
                        (build_element("input", &attrs, Nothing()))
                    };
                };
            };
        }

        build_element("input", &attrs, Nothing())
    }
}

#[allow(non_snake_case)]
pub fn PinInput(digits: u8) -> PreEscaped<String> {
    let js = r#"
    function pin_move_next(event) {
        const input = event.target;
        if (input.value.length === 1) {
            const nextInput = document.querySelector(`#${input.dataset.focusInputNext}`);
            if (nextInput) nextInput.focus();
        }
    }
    function pin_move_back(event) {
        const input = event.target;
        if (event.key === "Backspace" && input.value === "") {
            const prevInput = document.querySelector(`[data-focus-input-next='${input.id}']`);
            if (prevInput) {
                prevInput.focus();
                prevInput.setSelectionRange(1, 1);
            }
        }
    }
    function enforceNumericInput(event) {
        if (!/^[0-9]$/.test(event.key)) {
            event.preventDefault();
        }
    }
    "#;

    html! {
        div class="flex mb-2 space-x-2 rtl:space-x-reverse" {
            (script(js))

            @for i in 1..=digits {
                div {
                    label for=(format!("pin-{i}")) class="sr-only" { (format!("Pin {i}")) };

                    @if i == digits {
                        input type="text"
                        maxlength="1"
                        data-focus-input-init
                        pattern="[0-9]"
                        onkeypress="enforceNumericInput(event)"
                        id=(format!("pin-{i}"))
                        oninput="pin_move_next(event)" onkeydown="pin_move_back(event)"
                        class="block w-9 h-9 py-3 text-sm font-extrabold text-center text-gray-900 bg-white border border-gray-300 rounded-lg focus:ring-primary-500 focus:border-primary-500 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-primary-500 dark:focus:border-primary-500" required {};
                    } @else {
                    input type="text"
                    maxlength="1"
                    pattern="[0-9]"
                    data-focus-input-init
                    onkeypress="enforceNumericInput(event)"
                    data-focus-input-next=(format!("pin-{}", i+1))
                    id=(format!("pin-{i}"))
                    oninput="pin_move_next(event)" onkeydown="pin_move_back(event)"
                    class="block w-9 h-9 py-3 text-sm font-extrabold text-center text-gray-900 bg-white border border-gray-300 rounded-lg focus:ring-primary-500 focus:border-primary-500 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-primary-500 dark:focus:border-primary-500" required {};
                    }
                }
            }
    }
    }
}

pub struct NumberInputWidget {
    inner: Option<Box<dyn UIWidget>>,
    attr: HashMap<String, String>,
    buttons: bool,
}

#[allow(non_snake_case)]
pub fn NumberInput() -> NumberInputWidget {
    NumberInputWidget {
        inner: None,
        attr: HashMap::new(),
        buttons: false,
    }
}

impl NumberInputWidget {
    pub fn with_buttons(mut self) -> Self {
        self.buttons = true;
        self
    }

    pub fn min(self, min: u32) -> Self {
        self.add_attr("min", &min.to_string())
            .add_attr("data-input-counter-min", &min.to_string())
    }

    pub fn max(self, max: u32) -> Self {
        self.add_attr("max", &max.to_string())
            .add_attr("data-input-counter-max", &max.to_string())
    }

    pub fn step(self, step: u32) -> Self {
        self.add_attr("step", &step.to_string())
    }

    pub fn help<T: UIWidget + 'static>(mut self, help: T) -> Self {
        self.inner = Some(Box::new(help));
        self
    }
}

impl AttrExtendable for NumberInputWidget {
    fn add_attr(mut self, key: &str, val: &str) -> Self {
        self.attr.insert(key.into(), val.into());
        self
    }
}

impl InputAttr for NumberInputWidget {}

impl Render for NumberInputWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for NumberInputWidget {
    fn can_inherit(&self) -> bool {
        false
    }

    fn base_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn extended_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn render_with_class(&self, _: &str) -> Markup {
        let mut attrs = self.attr.clone();

        attrs.insert("type".to_string(), "text".to_string());
        attrs.insert("data-input-counter".into(), "".into());
        attrs.insert("class".into(), format!(
            "bg-gray-50 border-x-0 border-gray-300 h-11 font-medium text-center text-gray-900 text-sm focus:ring-blue-500 focus:border-blue-500 block w-full dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-blue-500 dark:focus:border-blue-500 {}", if self.inner.is_some() { "pb-6" } else { ""}));

        let input_name = attrs.get("id").map(|x| x.as_str()).unwrap_or_default();

        let input = build_element("input", &attrs, Nothing());

        html! {
            div class="relative flex items-center max-w-[11rem]" {
                @if self.buttons {
                button
                type="button"
                id="decrement-button"
                onclick=(format!("document.getElementById(\"{input_name}\").value = (+document.getElementById(\"{input_name}\").value || 0) + ((-document.getElementById(\"{input_name}\").step || 1) + 1);"))
                data-input-counter-decrement=(input_name)
                class="bg-gray-100 dark:bg-gray-700 dark:hover:bg-gray-600 dark:border-gray-600 hover:bg-gray-200 border border-gray-300 rounded-s-lg p-3 h-11 focus:ring-gray-100 dark:focus:ring-gray-700 focus:ring-2 focus:outline-none" {
                    svg class="w-3 h-3 text-gray-900 dark:text-white" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 18 2" {
                        path stroke="currentColor" stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M1 1h16" {};
                    };
                };
            };

                (input)

                @if let Some(inner) = &self.inner {
                div class="absolute bottom-1 start-1/2 -translate-x-1/2 rtl:translate-x-1/2 flex items-center text-xs text-gray-400 space-x-1 rtl:space-x-reverse" {
                    (inner)
                };
            };

                @if self.buttons {
                button type="button" id="increment-button"
                onclick=(format!("document.getElementById(\"{input_name}\").value = (+document.getElementById(\"{input_name}\").value || 0) + ((+document.getElementById(\"{input_name}\").step || 1) - 1);"))
                data-input-counter-increment=(input_name) class="bg-gray-100 dark:bg-gray-700 dark:hover:bg-gray-600 dark:border-gray-600 hover:bg-gray-200 border border-gray-300 rounded-e-lg p-3 h-11 focus:ring-gray-100 dark:focus:ring-gray-700 focus:ring-2 focus:outline-none" {
        svg class="w-3 h-3 text-gray-900 dark:text-white" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 18 18" {
            path stroke="currentColor" stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M9 1v16M1 9h16" {};
        };
                };
            };
            };
        }
    }
}

#[allow(non_snake_case)]
pub fn TimePicker(name: &str) -> DateTimePickerWidget {
    DateTimePickerWidget {
        attr: HashMap::new(),
        date: false,
    }
    .name(name)
}

#[allow(non_snake_case)]
pub fn DatePicker(name: &str) -> DateTimePickerWidget {
    DateTimePickerWidget {
        attr: HashMap::new(),
        date: true,
    }
    .name(name)
}

pub struct DateTimePickerWidget {
    attr: HashMap<String, String>,
    date: bool,
}

impl DateTimePickerWidget {
    pub fn min(self, min: &str) -> Self {
        self.add_attr("min", min)
    }

    pub fn max(self, max: &str) -> Self {
        self.add_attr("max", max)
    }
}

impl UIWidget for DateTimePickerWidget {
    fn can_inherit(&self) -> bool {
        false
    }

    fn base_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn extended_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn render_with_class(&self, _: &str) -> Markup {
        let mut attrs = self.attr.clone();

        attrs.insert("class".into(), "bg-gray-50 border leading-none border-gray-300 text-gray-900 text-sm rounded-lg focus:ring-blue-500 focus:border-blue-500 block w-full p-2.5 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-blue-500 dark:focus:border-blue-500".into());

        if self.date {
            attrs.insert("type".into(), "date".into());
        } else {
            attrs.insert("type".into(), "time".into());
        }

        let input = build_element("input", &attrs, Nothing());

        html! {
            div class="relative" {
                div class="absolute inset-y-0 end-0 top-0 flex items-center pe-3.5 pointer-events-none" {
                    @if !self.date {
                    svg class="w-4 h-4 text-gray-500 dark:text-gray-400" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" fill="currentColor" viewBox="0 0 24 24" {
                        path fill-rule="evenodd" d="M2 12C2 6.477 6.477 2 12 2s10 4.477 10 10-4.477 10-10 10S2 17.523 2 12Zm11-4a1 1 0 1 0-2 0v4a1 1 0 0 0 .293.707l3 3a1 1 0 0 0 1.414-1.414L13 11.586V8Z" clip-rule="evenodd" {};
                    };
                };
                };
                (input)
            };
        }
    }
}

impl Render for DateTimePickerWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl InputAttr for DateTimePickerWidget {}

impl AttrExtendable for DateTimePickerWidget {
    fn add_attr(mut self, key: &str, val: &str) -> Self {
        self.attr.insert(key.into(), val.into());
        self
    }
}

#[allow(non_snake_case)]
pub fn TextArea() -> TextAreaWidget {
    TextAreaWidget {
        content: String::new(),
        attr: HashMap::new(),
    }
}

pub struct TextAreaWidget {
    content: String,
    attr: HashMap<String, String>,
}

impl UIWidget for TextAreaWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn extended_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn render_with_class(&self, class: &str) -> Markup {
        let mut attrs = self.attr.clone();
        attrs.insert("class".into(), format!(
            "{class} block p-2.5 w-full text-sm text-gray-900 bg-gray-50 rounded-lg border border-gray-300 focus:ring-blue-500 focus:border-blue-500 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-blue-500 dark:focus:border-blue-500"
        ));

        build_element("textarea", &attrs, PreEscaped(self.content.clone()))
    }
}

impl Render for TextAreaWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl InputAttr for TextAreaWidget {}

impl AttrExtendable for TextAreaWidget {
    fn add_attr(mut self, key: &str, val: &str) -> Self {
        self.attr.insert(key.into(), val.into());
        self
    }
}

impl TextAreaWidget {
    pub fn content(mut self, content: String) -> Self {
        self.content = content;
        self
    }

    pub fn placeholder(self, placeholder: &str) -> Self {
        self.add_attr("placeholder", placeholder)
    }

    pub fn rows(self, rows: u32) -> Self {
        self.add_attr("rows", &rows.to_string())
    }

    pub fn pattern(self, pattern: &str) -> Self {
        self.add_attr("pattern", pattern)
    }

    pub fn maxlength(self, maxlength: &str) -> Self {
        self.add_attr("maxlength", maxlength)
    }

    pub fn minlength(self, minlength: &str) -> Self {
        self.add_attr("minlength", minlength)
    }
}

#[allow(non_snake_case)]
pub fn Select(
    label: Option<String>,
    id: &str,
    default: &str,
    options: Vec<(String, String)>,
) -> PreEscaped<String> {
    html! {
        @if let Some(label) = label {
            label for=(id) class="block mb-2 text-sm font-medium text-gray-900 dark:text-white"  { (label) };
        };

        select id=(id) class="bg-gray-50 border border-gray-300 text-gray-900 text-sm rounded-lg focus:ring-blue-500 focus:border-blue-500 block w-full p-2.5 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-blue-500 dark:focus:border-blue-500" {
            @for (value, label) in &options {
                @if value == default {
                    @if value.is_empty() {
                        option selected { (label) };
                    } @else {
                        option value=(value) selected { (label)  };
                    }
                } @else {
                    @if value.is_empty() {
                        option { (label) };
                    } @else {
                        option value=(value) { (label)  };
                    }
                };
            };
        };
    }
}

pub trait InputAttr: AttrExtendable + Sized {
    fn name(self, name: &str) -> Self {
        self.add_attr("name", name).add_attr("id", name)
    }

    fn value(self, value: &str) -> Self {
        self.add_attr("value", value)
    }

    fn readonly(self) -> Self {
        self.add_attr("readonly", "")
    }

    fn disabled(self) -> Self {
        self.add_attr("disabled", "")
    }

    fn required(self) -> Self {
        self.add_attr("required", "")
    }

    fn autofocus(self) -> Self {
        self.add_attr("autofocus", "")
    }

    fn autocomplete(self, value: bool) -> Self {
        self.add_attr("autocomplete", &value.to_string())
    }
}

#[allow(non_snake_case)]
pub fn HiddenInput(name: &str, value: &str) -> HiddenInputWidget {
    HiddenInputWidget {
        attrs: HashMap::new(),
    }
    .name(name)
    .value(value)
}

pub struct HiddenInputWidget {
    attrs: HashMap<String, String>,
}

impl AttrExtendable for HiddenInputWidget {
    fn add_attr(mut self, key: &str, val: &str) -> Self {
        self.attrs.insert(key.to_string(), val.to_string());
        self
    }
}

impl InputAttr for HiddenInputWidget {
    fn name(self, name: &str) -> Self {
        self.add_attr("name", name)
    }
}

impl Render for HiddenInputWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for HiddenInputWidget {
    fn can_inherit(&self) -> bool {
        false
    }

    fn base_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn extended_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn render_with_class(&self, _: &str) -> Markup {
        let mut attrs = self.attrs.clone();
        attrs.insert("type".into(), "hidden".into());
        build_element("input", &attrs, Nothing())
    }
}

#[allow(non_snake_case)]
pub fn FormResetButton<T: UIWidget + 'static>(inner: T) -> ButtonWidget {
    let btn = ButtonWidget {
        inner: Box::new(inner),
        attrs: HashMap::new(),
        color: None,
        hover_color: None,
    };
    btn.add_attr("type", "reset")
}

#[allow(non_snake_case)]
pub fn FormSubmitButton<T: UIWidget + 'static>(inner: T) -> ButtonWidget {
    let btn = ButtonWidget {
        inner: Box::new(inner),
        attrs: HashMap::new(),
        color: None,
        hover_color: None,
    };
    btn.add_attr("type", "submit")
}

pub struct ButtonWidget {
    inner: Box<dyn UIWidget>,
    attrs: HashMap<String, String>,
    color: Option<Box<dyn UIColor>>,
    hover_color: Option<Box<dyn UIColor>>,
}

impl ButtonWidget {
    pub fn color<C: UIColor + ColorCircle + 'static>(mut self, color: C) -> Self {
        let hover = color.next();
        self.color = Some(Box::new(color));
        self.hover_color = Some(Box::new(hover));
        self
    }
}

impl HTMXAttributes for ButtonWidget {}
impl InputAttr for ButtonWidget {}

impl AttrExtendable for ButtonWidget {
    fn add_attr(mut self, key: &str, val: &str) -> Self {
        self.attrs.insert(key.into(), val.into());
        self
    }
}

impl Render for ButtonWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for ButtonWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn extended_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn render_with_class(&self, class: &str) -> Markup {
        let mut attrs = self.attrs.clone();

        let color = if let Some(c) = &self.color {
            c.color_class()
        } else {
            "blue-700"
        };

        let hover_color = if let Some(c) = &self.hover_color {
            c.color_class()
        } else {
            "blue-800"
        };

        attrs.insert("class".to_string(), format!(
            "{class} px-5 py-2.5 text-sm font-medium text-white inline-flex items-center bg-{color} hover:bg-{hover_color} rounded-lg text-center"
        ));

        build_element("button", &attrs, self.inner.render())
    }
}

#[allow(non_snake_case)]
pub fn Button<T: UIWidget + 'static>(inner: T) -> ButtonWidget {
    ButtonWidget {
        inner: Box::new(inner),
        attrs: HashMap::new(),
        color: None,
        hover_color: None,
    }
}

#[allow(non_snake_case)]
pub fn ButtonGroup(buttons: Vec<ButtonWidget>) -> PreEscaped<String> {
    html! {
        div class="inline-flex rounded-md shadow-xs" role="group" {
                @for (index, element) in buttons.iter().enumerate() {
                    @if index == 0 {
                    button type="button" class="px-4 py-2 text-sm font-medium text-gray-900 bg-white border border-gray-200 rounded-s-lg hover:bg-gray-100 hover:text-blue-700 focus:z-10 focus:ring-2 focus:ring-blue-700 focus:text-blue-700 dark:bg-gray-800 dark:border-gray-700 dark:text-white dark:hover:text-white dark:hover:bg-gray-700 dark:focus:ring-blue-500 dark:focus:text-white" {
                        (element.inner)
                    }
                    } @else if index == (buttons.len()-1) {
                        button type="button" class="px-4 py-2 text-sm font-medium text-gray-900 bg-white border border-gray-200 rounded-e-lg hover:bg-gray-100 hover:text-blue-700 focus:z-10 focus:ring-2 focus:ring-blue-700 focus:text-blue-700 dark:bg-gray-800 dark:border-gray-700 dark:text-white dark:hover:text-white dark:hover:bg-gray-700 dark:focus:ring-blue-500 dark:focus:text-white" {
                            (element.inner)
                        }
                    } @else {
                        button type="button" class="px-4 py-2 text-sm font-medium text-gray-900 bg-white border-t border-b border-gray-200 hover:bg-gray-100 hover:text-blue-700 focus:z-10 focus:ring-2 focus:ring-blue-700 focus:text-blue-700 dark:bg-gray-800 dark:border-gray-700 dark:text-white dark:hover:text-white dark:hover:bg-gray-700 dark:focus:ring-blue-500 dark:focus:text-white" {
                            (element.inner)
                        }
                };
            };
        };
    }
}
