use maud::{Markup, PreEscaped, Render, html};

use crate::ui::UIWidget;

use super::{div::Div, space::ScreenValue};

#[allow(non_snake_case)]
pub fn Table<T: UIWidget + 'static + Clone>(inner: Vec<Vec<T>>) -> TableWidget {
    let inner = Div().vanish().push_for_each(&inner, |row| {
        TableRow(
            Div()
                .vanish()
                .push_for_each(&row, |col| TableData(col.clone())),
        )
    });

    TableWidget(Box::new(inner), Vec::new(), None, None)
}

#[allow(non_snake_case)]
pub fn TableRaw<T: UIWidget + 'static + Clone>(inner: T) -> TableWidget {
    TableWidget(Box::new(inner), Vec::new(), None, None)
}

pub struct TableWidget(
    Box<dyn UIWidget>,
    Vec<String>,
    Option<Box<dyn UIWidget>>,
    Option<Caption>,
);

impl TableWidget {
    pub fn header<T: UIWidget + 'static>(mut self, header: T) -> Self {
        self.2 = Some(Box::new(header));
        self
    }

    pub fn caption(mut self, caption: Caption) -> Self {
        self.3 = Some(caption);
        self
    }

    pub fn border_collapse(mut self) -> Self {
        self.1.push("border-collapse".to_string());
        self
    }

    pub fn border_seperate(mut self) -> Self {
        self.1.push("border-separate".to_string());
        self
    }

    pub fn border_spacing(mut self, spacing: ScreenValue) -> Self {
        self.1
            .push(format!("border-spacing-{}", spacing.to_value()));
        self
    }

    pub fn border_spacing_x(mut self, spacing: ScreenValue) -> Self {
        self.1
            .push(format!("border-spacing-x-{}", spacing.to_value()));
        self
    }

    pub fn border_spacing_y(mut self, spacing: ScreenValue) -> Self {
        self.1
            .push(format!("border-spacing-y-{}", spacing.to_value()));
        self
    }

    pub fn layout_fixed(mut self) -> Self {
        self.1.push("table-fixed".to_string());
        self
    }

    pub fn layout_auto(mut self) -> Self {
        self.1.push("table-auto".to_string());
        self
    }
}

impl Render for TableWidget {
    fn render(&self) -> maud::Markup {
        self.render_with_class("")
    }
}

impl UIWidget for TableWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        self.1.clone()
    }

    fn extended_class(&self) -> Vec<String> {
        self.base_class()
    }

    fn render_with_class(&self, class: &str) -> maud::Markup {
        html! {
            table class=(format!("{} {class}", self.base_class().join(" "))) {
                @if let Some(caption) = &self.3 {
                    (caption)
                }

                @if let Some(header) = &self.2 {
                    thead {
                        (header)
                    };
                };

                (self.0.as_ref())
            };
        }
    }
}

pub struct Caption(Box<dyn UIWidget>, bool);

impl Caption {
    #[allow(non_snake_case)]
    pub fn Top<T: UIWidget + 'static>(inner: T) -> Self {
        Self(Box::new(inner), true)
    }

    #[allow(non_snake_case)]
    pub fn Bottom<T: UIWidget + 'static>(inner: T) -> Self {
        Self(Box::new(inner), false)
    }
}

impl Render for Caption {
    fn render(&self) -> maud::Markup {
        self.render_with_class("")
    }
}

impl UIWidget for Caption {
    fn can_inherit(&self) -> bool {
        false
    }

    fn base_class(&self) -> Vec<String> {
        if self.1 {
            vec!["caption-top".to_string()]
        } else {
            vec!["caption-bottom".to_string()]
        }
    }

    fn extended_class(&self) -> Vec<String> {
        self.base_class()
    }

    fn render_with_class(&self, _: &str) -> maud::Markup {
        html! {
            caption class=(self.base_class().join(" ")) {
                (self.0.as_ref())
            };
        }
    }
}

macro_rules! element_widget {
    ($name:ident, $widget:ident, $element:ident) => {
        #[allow(non_snake_case)]
        pub fn $name<T: UIWidget + 'static>(inner: T) -> $widget {
            $widget(Box::new(inner))
        }

        pub struct $widget(Box<dyn UIWidget>);

        impl Render for $widget {
            fn render(&self) -> Markup {
                self.render_with_class("")
            }
        }

        impl UIWidget for $widget {
            fn can_inherit(&self) -> bool {
                true
            }

            fn base_class(&self) -> Vec<String> {
                Vec::new()
            }

            fn extended_class(&self) -> Vec<String> {
                let mut c = self.base_class();
                c.extend_from_slice(&self.0.extended_class());
                c
            }

            fn render_with_class(&self, class: &str) -> Markup {
                html! {
                        $element class=(class) {
                            (self.0.as_ref())
                        }
                }
            }
        }
    };
}

element_widget!(TableRow, TableRowWidget, tr);
element_widget!(TableHead, TableHeadWidget, th);
element_widget!(TableData, TableDataWidget, td);
element_widget!(Header, HeaderWidget, header);

// TODO : adv tables
// table options
// aggregate row

pub struct AdvancedTable {
    header: Option<Vec<PreEscaped<String>>>,
    caption: Option<PreEscaped<String>>,
    rows: Vec<Vec<PreEscaped<String>>>,
    hover: bool,
}

impl AdvancedTable {
    pub fn hover(mut self) -> Self {
        self.hover = true;
        self
    }

    pub fn caption<T: UIWidget + 'static>(mut self, caption: T) -> Self {
        self.caption = Some(caption.render());
        self
    }

    pub fn header(mut self, row: Vec<PreEscaped<String>>) -> Self {
        self.header = Some(row);
        self
    }
}

impl Render for AdvancedTable {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for AdvancedTable {
    fn can_inherit(&self) -> bool {
        false
    }

    fn base_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn extended_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn render_with_class(&self, _: &str) -> Markup {
        html! {
                div class="relative overflow-x-auto shadow-md sm:rounded-lg" {
                    table class="w-full text-sm text-left rtl:text-right text-gray-500 dark:text-gray-400" {
                        @if let Some(cap) = &self.caption {
                            caption class="p-5 text-lg font-semibold text-left rtl:text-right text-gray-900 bg-white dark:text-white dark:bg-gray-800" {
                                (cap)
                            };
                        }

                        @if let Some(header) = &self.header {
                        thead class="text-xs text-gray-700 uppercase bg-gray-50 dark:bg-gray-700 dark:text-gray-400" {
                            tr {
                                @for h in header {
                                    th scope="col" class="px-6 py-3" {
                                        (h);
                                    }
                                }
                            };
                        };
                    };
                        tbody {
                            @for row in &self.rows {
                                tr class=(format!("bg-white border-b dark:bg-gray-800 dark:border-gray-700 border-gray-200 {}", if self.hover { "hover:bg-gray-50 dark:hover:bg-gray-600" } else { "" })) {
                                    @for cell in row {
                                        td scope="row" class="px-6 py-4" {
                                            (cell)
                                        }
                                    }
                                }
                            };
                    };
                };
            }
        }
    }
}

// selectable rows

// sortable cols

#[allow(non_snake_case)]
pub fn SortableIcon() -> PreEscaped<String> {
    html! {
            svg class="w-3 h-3 ms-1.5" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" fill="currentColor" viewBox="0 0 24 24" {
        path d="M8.574 11.024h6.852a2.075 2.075 0 0 0 1.847-1.086 1.9 1.9 0 0 0-.11-1.986L13.736 2.9a2.122 2.122 0 0 0-3.472 0L6.837 7.952a1.9 1.9 0 0 0-.11 1.986 2.074 2.074 0 0 0 1.847 1.086Zm6.852 1.952H8.574a2.072 2.072 0 0 0-1.847 1.087 1.9 1.9 0 0 0 .11 1.985l3.426 5.05a2.123 2.123 0 0 0 3.472 0l3.427-5.05a1.9 1.9 0 0 0 .11-1.985 2.074 2.074 0 0 0-1.846-1.087Z" {};
        }
    }
}

// filterable tables
// wrap data to table (crud)
// table pagination
